#include "qanimtt.h"

#include <qapplication.h>
#include <qpainter.h>
#include <qbitmap.h>
#include <qtimer.h>
#include <qcursor.h>

static const int DEFAULT_FRAMES = 30;

QAnimTT::QAnimTT( QWidget * parent )
    : QWidget( parent, "animtt" ),
      m_animPix( "aim.png" ),
      m_rotate( 0 ),
      m_shearX( 0 ),
      m_shearY( 0 ),
      m_return( false ),
      m_dirty( false )
{
    setFocusPolicy( NoFocus );
    setBackgroundMode( NoBackground );
    move( QCursor::pos() );
    resize( 0, 0 );
    hide();
    connect( &m_frameTimer, SIGNAL(timeout()), SLOT(internalUpdate()) );
}

void QAnimTT::display( const QString & message, Effect eff, int durationMs )
{
    m_message = message;
    m_effect = eff;
    displayInternal();

    m_frameTimer.start( 1000/DEFAULT_FRAMES );

    // show widget and schedule a repaint
    show();
    update();

    // close the message window after given mS
    if ( durationMs > 0 )
    {
        connect( &m_timer, SIGNAL( timeout() ), SLOT( hide() ) );
        m_timer.start( durationMs, true );
    } else
        m_timer.stop();
}

void QAnimTT::paintEvent( QPaintEvent * e )
{
    if ( m_dirty )
    {
        displayInternal();
        m_dirty = false;
    }
    QPainter p( this );
    p.drawPixmap( e->rect().topLeft(), m_pixmap, e->rect() );
}

void QAnimTT::mousePressEvent( QMouseEvent * /*e*/ )
{
    m_timer.stop();
    m_frameTimer.stop();
    hide();
}


QPixmap QAnimTT::scale()
{
    QWMatrix matrix;

    matrix = matrix.translate( width()/2, height()/2 );
    QPixmap pixmap = m_animPix.xForm(
        matrix.scale( m_shearX, m_shearY ) );

    if ( m_shearX >=  1)
    {
        if ( !m_return )
        {
            m_return = true;
        }
    }

    if ( m_shearX <= 0 )
    {
        if ( m_return )
            m_return = false;
    }

    if ( m_return )
    {
        m_shearX -= .02;
        m_shearY -= .02;
    }
    else
    {
        m_shearX += .02;
        m_shearY += .02;
    }

    return pixmap;
}

QPixmap QAnimTT::shear()
{
    QWMatrix matrix;
    //matrix = matrix.translate( width()/2, height()/2 );

    QPixmap pixmap = m_animPix.xForm( matrix.shear( m_shearX, -m_shearY ) );

    if ( m_shearX >=  1)
    {
        if ( !m_return )
        {
            m_return = true;
        }
    }

    if ( m_shearX <= 0 )
    {
        if ( m_return )
            m_return = false;
    }

    if ( m_return )
    {
        m_shearX -= .02;
        m_shearY -= .02;
    }
    else
    {
        m_shearX += .02;
        m_shearY += .02;
    }
    return pixmap;
}

QPixmap QAnimTT::rotate()
{
    QWMatrix matrix;
    matrix = matrix.translate( width()/2, height()/2 );
    //double tr = ( double )m_rotate / ( double )360;
    QPixmap pixmap = m_animPix.xForm(
        matrix.rotate( m_rotate ) );
    m_rotate += 5;
    if ( m_rotate >= 360 )
        m_rotate = 0;
    return pixmap;
}

void QAnimTT::displayInternal()
{
     // determine text rectangle
    QRect textRect = fontMetrics().boundingRect( m_message );
    textRect.moveBy( -textRect.left(), -textRect.top() );
    textRect.addCoords( 0, 0, 2, 2 );
    int width = textRect.width(),
        height = textRect.height(),
        textXOffset = 0,
        shadowOffset = m_message.isRightToLeft() ? -1 : 1;


    QPixmap symbol;
    switch( m_effect )
    {
    case Scale:
        symbol = scale();
        break;
    case Shear:
        symbol = shear();
        break;
    case Rotate:
        symbol = rotate();
        break;
    default:
        symbol = m_animPix;
        break;
    }
    textXOffset = 2 + symbol.width();
    width += textXOffset;
    height = QMAX( height, symbol.height() );

    QRect geometry( 0, 0, width + 10, height + 8 );

    // resize pixmap, mask and widget
    static QBitmap mask;
    mask.resize( geometry.size() );
    m_pixmap.resize( geometry.size() );
    resize( geometry.size() );

    // create and set transparency mask
    QPainter maskPainter( &mask);
    mask.fill( Qt::black );
    maskPainter.setBrush( Qt::white );
    maskPainter.drawRoundRect( geometry, 1600 / geometry.width(), 1600 / geometry.height() );
    setMask( mask );

    // draw background
    QPainter bufferPainter( &m_pixmap );
    bufferPainter.setPen( Qt::black );
    bufferPainter.setBrush( backgroundColor() );
    bufferPainter.drawRoundRect( geometry, 1600 / geometry.width(), 1600 / geometry.height() );

    // draw icon if present
    if ( !symbol.isNull() )
        bufferPainter.drawPixmap( 5, 4, symbol, 0, 0, symbol.width(), symbol.height() );

    // draw shadow and text
    int yText = ( geometry.height() - textRect.height() ) / 2;
    bufferPainter.setPen( backgroundColor().dark( 115 ) );
    bufferPainter.drawText( 5 + textXOffset + shadowOffset, yText + 1,
                            textRect.width(), textRect.height(), AlignCenter, m_message );
    bufferPainter.setPen( foregroundColor() );
    bufferPainter.drawText( 5 + textXOffset, yText,
                            textRect.width(), textRect.height(), AlignCenter, m_message );
}

void QAnimTT::internalUpdate()
{
    m_dirty = true;
    update();
}

void QAnimTT::hide()
{
    QWidget::hide();
    qApp->exit();
}
